package com.simplicite.objects.Demo;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.util.Map;

import org.junit.Test;

import com.simplicite.util.Action;
import com.simplicite.util.AppLog;
import com.simplicite.util.Grant;
import com.simplicite.util.Message;
import com.simplicite.util.ObjectDB;
import com.simplicite.util.ObjectField;
import com.simplicite.util.PrintTemplate;
import com.simplicite.util.Tool;
import com.simplicite.util.annotations.BusinessObjectAction;
import com.simplicite.util.annotations.BusinessObjectPublication;
import com.simplicite.util.tools.BarcodeTool;
import com.simplicite.util.tools.DocxTool;
import com.simplicite.util.tools.MailTool;
import com.simplicite.util.tools.JUnitTool;

/**
 * Product business object
 */
public class DemoProduct extends ObjectDB {
	private static final long serialVersionUID = 1L;

	/** Default increment */
	public static final int DEFAULT_INCREMENT = 10;

	public static final String REFERENCE_FIELDNAME = "demoPrdReference";
	public static final String STOCK_FIELDNAME = "demoPrdStock";
	public static final String INCREMENT_FIELDNAME = "demoPrdIncrement";

	/** Init default increment */
	@Override
	public void initAction(Action action) {
		if ("DEMO_INCSTOCK".equals(action.getName())) {
			ObjectField f = action.getConfirmField(getGrant().getLang(), INCREMENT_FIELDNAME);
			if (f!=null) {
				String def = String.valueOf(DEFAULT_INCREMENT);
				f.setDefaultValue(def);
			}
		}
	}

	/** Action: increase stock */
	@BusinessObjectAction
	public String increaseStock(Map<String, String> params) {
		try {
			int inc = Tool.parseInt(params.get(INCREMENT_FIELDNAME), DEFAULT_INCREMENT);
			if (inc > 0) {
				ObjectField s = getField(STOCK_FIELDNAME);
				s.setValue(s.getInt(0) + inc);
				getTool().validateAndSave();
				// Log
				AppLog.log("DEMO_INFO", getClass(), "increaseStock", "Stock for " + getFieldValue(REFERENCE_FIELDNAME) + " is now " + s.getValue(), getGrant());
				// User message
				return Message.formatSimpleInfo("DEMO_PRD_STOCK_INCREASED:" + s.getValue());
			} else {
				return Message.formatSimpleError("DEMO_PRD_ERR_INCREMENT:" + inc);
			}
		} catch (Exception e) {
			return Message.formatSimpleError(e);
		}
	}

	/** Action: decrease stock */
	@BusinessObjectAction
	public String decreaseStock() {
		try {
			int dec = getIntParameter("QUANTITY", 0);
			ObjectField s = getField(STOCK_FIELDNAME);
			s.setValue(s.getInt() - dec);
			getTool().validateAndSave();
			// Log
			AppLog.log("DEMO_INFO", getClass(), "decreaseStock", "Stock for " + getFieldValue(REFERENCE_FIELDNAME) + " is now " + s.getValue(), getGrant());
			// User message
			return Message.formatSimpleInfo("DEMO_PRD_STOCK_DECREASED:" + dec);
		} catch (Exception e) {
			return Message.formatSimpleError(e);
		}
	}

	/** Action method: send product data in an email */
	@BusinessObjectAction
	public String sendEmail(Action a) {
		try {
			MailTool mt = new MailTool(getGrant());
			mt.addRcpt(getGrant().getEmail());
			String ref = getFieldValue(REFERENCE_FIELDNAME);
			mt.setSubject(getName() + " " + ref);
			mt.addAttach(this, getField("demoPrdBrochure"));
			String picCid = mt.addImage(this, getField("demoPrdPicture"));
			mt.setBody(
				"<h1>" + Tool.toHTML(ref) + "</h1>" +
				"<img src=\"cid:" + picCid +  "\"/>" +
				"<h3>" + Tool.toHTML(getFieldValue("demoPrdName")) + "</h3>" +
				"<h5>" + Tool.toHTML(getFieldValue("demoPrdDescription")) + "</h5>" +
				"<div>" + getFieldValue("demoPrdDocumentation") + "</div>"
			);
			mt.send();
			return Message.formatSimpleInfo("OK");
		} catch (Exception e) {
			AppLog.error(null, e, getGrant());
			return Message.formatSimpleError(e.getMessage());
		}
	}

	/** Publication: Microsoft Word(R) catalog */
	@BusinessObjectPublication
	public Object printCatalog(PrintTemplate pt) {
		try {
			DocxTool dt = new DocxTool();
			dt.newDocument();
			dt.addStyledParagraph(DocxTool.STYLE_TITLE, getFieldValue("demoPrdName") + " (" + getFieldValue(REFERENCE_FIELDNAME) + ")");
			dt.addParagraph(getFieldValue("demoPrdDescription"));
			dt.addHTML(getFieldValue("demoPrdDocumentation"));
			return dt.toByteArray();
		} catch (Exception e) {
			AppLog.error("Unable to publish " + pt.getName(), e, getGrant());
			return e.getMessage();
		}
	}

	@Override
	public String preSave() {
		// Manage the EAN13 image
		ObjectField ean13 = getField("demoPrdEan13");
		ObjectField ean13Img = getField("demoPrdEan13Image");
		if (ean13.isEmpty())
			ean13Img.setValue(""); // Remove image if EAN13 is empty
		else if (isNew() || ean13.hasChanged())
			ean13Img.setDocument(this, ean13.getValue() + ".png", BarcodeTool.ean13Image(ean13.getValue())); // (Re)generate image if ENA13 has changed
		return null;
	}

	/** Hook override: custom short label */
	@Override
	public String getUserKeyLabel(String[] row) {
		return getFieldValue(REFERENCE_FIELDNAME, row);
	}

	/** Hook override: hide history records on tree view */
	@Override
	public boolean canReference(String objName, String fkFieldName) {
		return !isTreeviewInstance() || "DemoProductHistoric".equals(objName);
	}

	/** JUnit test class */
	public static class DemoProductTest {
		/** Decrement test */
		@Test
		public void testDecrement() {
			try {
				ObjectDB prd = Grant.getSystemAdmin().getTmpObject("DemoProduct");
				prd.setValues(prd.search().get(0), true);
				ObjectField f = prd.getField(STOCK_FIELDNAME);
				int n = f.getInt(0);
				prd.setParameter("QUANTITY", 10);
				prd.invokeAction("DEMO_DECSTOCK");
				assertEquals((long)n - 10, f.getInt(0));
			} catch (Exception e) {
				fail(e.getMessage());
			}
		}
	}

	/** Hook override: launch JUnit tests classes */
	@Override
	public String unitTests() {
		JUnitTool jut = new JUnitTool(getGrant());
		return
			jut.run("com.simplicite.tests.Demo.DemoTests") + // Shared code unit tests class
			jut.run(DemoProductTest.class); // Nested test class
	}
}
